@@ -5,7 +5,7 @@ use warnings;
 
 use parent qw(IO::Async::Notifier);
 
-our $VERSION = '0.022';
+our $VERSION = '0.023';
 
 =head1 NAME
 
@@ -13,7 +13,7 @@ Net::Async::AMQP - provides client interface to AMQP using L<IO::Async>
 
 =head1 VERSION
 
-version 0.022
+version 0.023
 
 =head1 SYNOPSIS
 
@@ -54,6 +54,8 @@ The following AMQP features are supported:
 
 =item * QoS
 
+=item * SSL
+
 =back
 
 =head2 RabbitMQ-specific features
@@ -72,6 +74,9 @@ RabbitMQ provides some additional features:
 
 =head2 Missing features
 
+The following features aren't currently implemented - raise a request via RT or by email (L</AUTHOR>)
+if you want any of these:
+
 =over 4
 
 =item * Transactions
